Package: FastGaSP
Type: Package
Title: Fast and Exact Computation of Gaussian Stochastic Process
Version: 0.6.4
Date: 2026-05-12
Authors@R: c(person(given="Mengyang", family="Gu", role=c("aut", "cre"),   email="mengyang@pstat.ucsb.edu"),
  person(given="Xinyi", family="Fang", role=c("aut"), 
  email="xinyifang@ucsb.edu"),
  person(given="Yizi", family="Lin", role=c("aut"), 
  email="lin768@ucsb.edu"))
Maintainer: Mengyang Gu <mengyang@pstat.ucsb.edu>
Author: Mengyang Gu [aut, cre],
  Xinyi Fang [aut],
  Yizi Lin [aut]
Description: Implements fast and exact computation of Gaussian stochastic process with the Matern kernel using forward filtering and backward smoothing algorithm. It includes efficient implementations of the inverse Kalman filter, with applications such as estimating particle interaction functions. These tools support models with or without noise. Additionally, the package offers algorithms for fast parameter estimation in latent factor models, where the factor loading matrix is orthogonal, and latent processes are modeled by Gaussian processes.  See the references: 1) Mengyang Gu and Yanxun Xu (2020), Journal of Computational and Graphical Statistics; 2) Xinyi Fang and Mengyang Gu (2024), <doi:10.48550/arXiv.2407.10089>; 3) Mengyang Gu and Weining Shen (2020), Journal of Machine Learning Research; 4) Yizi Lin, Xubo Liu, Paul Segall and Mengyang Gu (2025), <doi:10.48550/arXiv.2501.01324>.
License: GPL (>= 2)
Depends: methods
Imports: Rcpp,rstiefel
LinkingTo: Rcpp, RcppEigen
NeedsCompilation: yes
Repository: CRAN
Packaged: 2026-05-12 18:09:12 UTC; mengyanggu
RoxygenNote: 7.3.2
Date/Publication: 2026-05-13 07:30:56 UTC
Built: R 4.6.0; aarch64-apple-darwin23; 2026-05-13 10:16:05 UTC; unix
Archs: FastGaSP.so.dSYM
