useDynLib(FastGaSP)

# Import
import("Rcpp", "methods")
importFrom("Rcpp", "evalCpp")
importFrom("stats","rnorm","qnorm","sd","optim","optimize","runif","var")
importFrom("rstiefel", "rustiefel")
# Exports
export(fgasp,predict.fgasp,Get_log_det_S2,Construct_W0_matern_5_2, Construct_W0_matern_3_2,Construct_W0_exp,Construct_G_matern_5_2, Construct_G_matern_3_2,Construct_G_exp,Construct_W_matern_5_2, Construct_W_matern_3_2,Construct_W_exp,log_lik,Sample_KF,Sample_KF_post,Get_C_R_K_Q,fmou,gppca,simulate_particle,trajectory_data,extract_time_window,unnormalized_Vicsek,f_Vicsek_variation,Get_Q_K,Get_R_y,IKF_CG_particle,A_times_x_particle,A_t_times_x_particle,get_consecutive_data,IKF_CG_particle_cell,IKF)
exportMethods(show,predict,fit.fmou,fit.gppca,predict.fmou,predict.gppca, fit)
exportClasses(fgasp,predictobj.fgasp,fmou,gppca,particle.data, particle.est)

